import gc
import unittest
import subprocess

from comsdk.graph import *
from comsdk.parser import Parser

path_to_comsdk = "/home/lbstr/bmstu/comsdk"
path_to_pycomsdk = "/home/lbstr/bmstu/pycomsdk"


class TestAdvancedParser(unittest.TestCase):
    def setUp(self):
        self.test_files_dir = os.path.join(os.path.dirname(__file__), "test_adot_files")

    def test_node_attributes_parsing(self):
        """Проверка парсинга атрибутов узлов"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "node_attrs.adot"))

        # Проверка атрибутов состояния
        state = graph.init_state
        self.assertEqual(state.parallelization_policy.__class__.__name__, "ThreadParallelizationPolicy")
        self.assertIsNotNone(state.selector)
        self.assertEqual(state.comment, "Test node with attributes")

    def test_edge_types_parsing(self):
        """Проверка обработки разных типов ребер"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "edge_types.adot"))

        # Проверка обычного и многопоточного ребер
        transfers = graph.init_state.transfers
        self.assertEqual(transfers[0].edge.order, 0)  # ->
        self.assertEqual(transfers[1].edge.order, 1)  # =>

    def test_subgraph_parsing(self):
        """Тестирование встраивания подграфов"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "subgraph_test.adot"))

        # Проверка замены состояния подграфом
        state = graph.init_state
        self.assertTrue(hasattr(state, "_proxy_state"))
        self.assertEqual(len(state._proxy_state.transfers), 2)

    def test_keys_mapping(self):
        """Проверка преобразования ключей"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "keys_mapping.adot"))

        # Проверка настроек ключей для обработчика
        edge = graph.init_state.transfers[0].edge
        self.assertEqual(edge._io_mapping._keys_mapping["local_key"], "global.key.path")

    def test_config_integration(self):
        """Интеграция с внешними конфигурационными файлами"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "config_integration.adot"))

        # Проверка загрузки параметров из конфига
        func = graph.init_state.transfers[0].edge.morph_f
        self.assertEqual(func.module, "configured_module")
        self.assertEqual(func.name, "configured_function")

    def test_error_handling(self):
        """Проверка обработки ошибок"""
        parser = Parser()

        with self.assertRaises(Exception) as context:
            parser.parse_file(os.path.join(self.test_files_dir, "invalid_syntax.adot"))

        self.assertIn("Syntax error", str(context.exception))

    def test_pre_post_processors(self):
        """Тестирование пре- и пост-процессоров"""
        parser = Parser()
        graph = parser.parse_file(os.path.join(self.test_files_dir, "pre_post_processors.adot"))

        edge = graph.init_state.transfers[0].edge
        self.assertIsNotNone(edge.preprocess)
        self.assertIsNotNone(edge.postprocess)


class ParserGoodCheck(unittest.TestCase):


    def test_trivial_graph(self):
        parsr = Parser()

        # Проверка загрузки файла
        self.assertTrue(os.path.exists("./adot/trivial_v2.adot"))
        gr = parsr.parse_file("./adot/trivial_v2.adot")
        print(gr)

        # Основной сценарий
        data = {"a": 1}
        gr.run(data)
        self.assertEqual(data["a"], 4)

        # Проверка повторного использования
        data = {"a": 4}
        gr.run(data)
        self.assertEqual(data["a"], 7)

    def test_branching_graph(self):
        parsr = Parser()
        gr = parsr.parse_file("./adot/branching.adot")
        data = {"a": 1, "b": 1}
        gr.run(data)
        self.assertEqual(data["a"], 4)
        self.assertEqual(data["b"], 4)

    def test_cycled_graph(self):
        parsr = Parser()
        gr = parsr.parse_file("./adot/cycled.adot")
        data = {"a": 10}
        gr.run(data)
        self.assertEqual(data["a"], 0)

    def test_complex_graph(self):
        parsr = Parser()
        gr = parsr.parse_file("./adot/complex.adot")
        data = {"a": 1, "b": 1}
        gr.run(data)
        self.assertEqual(data["a"], 4)
        self.assertEqual(data["b"], 4)

    def test_cpp_trivial_graph(self):
        parsr = Parser(tocpp=True)
        gr = parsr.parse_file("./tests/adot/cpptrivial.adot")
        parsr.generate_cpp(path_to_comsdk+"res.cpp")
        command = "cd "+path_to_comsdk+"; "+path_to_pycomsdk+"/cpp/run.sh "+path_to_comsdk+"res.cpp"
        subprocess.check_output(["bash", "-c", command])
    
    def test_cpp_branching_graph(self):
        parsr = Parser(tocpp=True)
        gr = parsr.parse_file("./tests/adot/cppbranching.adot")
        parsr.generate_cpp(path_to_comsdk+"res.cpp")
        command = "cd "+path_to_comsdk+"; "+path_to_pycomsdk+"/cpp/run.sh "+path_to_comsdk+"res.cpp"
        subprocess.check_output(["bash", "-c", command])

    def test_cpp_cycled_graph(self):
        parsr = Parser(tocpp=True)
        gr = parsr.parse_file("./tests/adot/cppcycled.adot")
        parsr.generate_cpp(path_to_comsdk+"res.cpp")
        command = "cd "+path_to_comsdk+"; "+path_to_pycomsdk+"/cpp/run.sh "+path_to_comsdk+"res.cpp"
        subprocess.check_output(["bash", "-c", command])

    def test_cpp_complex_graph(self):
        parsr = Parser(tocpp=True)
        gr = parsr.parse_file("./tests/adot/cppcomplex.adot")
        parsr.generate_cpp(path_to_comsdk+"res.cpp")
        command = "cd "+path_to_comsdk+"; "+path_to_pycomsdk+"/cpp/run.sh "+path_to_comsdk+"res.cpp"
        subprocess.check_output(["bash", "-c", command])

if __name__ == '__main__':
    unittest.main()

